import DB from './DB';
import QueryBuilder from './QueryBuilder';

// Representa un modelos de base de datos
export default class Model {
  static get databaseName() {
    throw new Error('databaseName not defined');
  }
  static get tableName() {
    throw new Error('tableName not defined');
  }
  static get primaryKey() {
    return "id";
  }
  constructor(props = {}) {
    // Copiar las propiedades al objeto actual (this)
    Object.assign(this, props);
  }

  //constructor(public id: number, public name: string) {}

  // Función que convierte valores de la base de datos a propiedades de la entidad
  static databaseToModel(databaseValues) {
    return new this(databaseValues);
  }

  // Función que convierte un modelo a valores que pueden ser insertados en la base de datos
  static modelToDatabase(props) {
    return Object.assign({}, props);
  }

  /**
   * ```js
   * Animal.executeSql("SELECT * FROM tb_animal WHERE id = ?", [7])
   * ```
   * @return {QueryResult} result
   */
  static executeSql(sql, args = []) {
    const db = DB.get(this.databaseName);
    return db.executeSql(sql, args);
  }
  static rawQuery(sql, args = []) {
    const db = DB.get(this.databaseName);
    return db.rawQuery(sql, args);
  }

  /**
   * ```js
   * Animal.findBy<Animal>("color", "LIKE", '%Brown%')
   * ```
   * @param column mombre de la columna
   * @param op operador: =, <>, <, <=, >, >=, LIKE
   * @param value qriterio de busqueda
   * @return registro
   */
  static async findBy(column, op, value) {
    const sql = `
      SELECT * FROM ${this.tableName} WHERE ${column} ${op} ?
    `;
    const {
      rows
    } = await this.executeSql(sql, [value]);
    return rows.length ? this.databaseToModel(rows[0]) : undefined;
  }

  /**
   * ```js
   * Animal.find(1)
   * ```
   * @param id identificador del registro
   * @return registro
   */
  static find(id) {
    return this.findBy(this.primaryKey, "=", id);
  }

  /**
   * ```js
   * Animal.first()
   * ```
   * @return primer registro
   */
  static async first() {
    const sql = `
      SELECT * FROM ${this.tableName} ORDER BY ROWID ASC LIMIT 1
    `;
    const {
      rows
    } = await this.executeSql(sql);
    return rows.length ? this.databaseToModel(rows[0]) : null;
  }

  /**
   * ```js
   * Animal.last()
   * ```
   * @return último registro
   */
  static async last() {
    const sql = `
      SELECT * FROM ${this.tableName} ORDER BY ROWID DESC LIMIT 1
    `;
    const {
      rows
    } = await this.executeSql(sql);
    return rows.length ? this.databaseToModel(rows[0]) : null;
  }

  /**
   * ```js
   * Animal.count()
   * ```
   * @return numero de registros
   */
  static async count() {
    const sql = `
      SELECT COUNT(*) AS __count__ FROM ${this.tableName}
    `;
    const result = await this.executeSql(sql);
    const row = result.rows[0];
    return row["__count__"];
  }

  /**
   * ```js
   * Animal.all()
   * ```
   * @return array con todos los registros
   */
  static async all() {
    const sql = `
      SELECT * FROM ${this.tableName}
    `;
    const result = await this.rawQuery(sql);
    const list = [];
    for (let i = 0; i < result.rows.length; i++) {
      const row = result.rows.item(i);
      list.push(this.databaseToModel(row));
    }
    return list;
  }

  /**
   * ```js
   * Animal.query({
   *   columns: 'id, name',
   *   where: {
   *     clause: 'age > ? AND age < ?',
   *     args: [ 2, 10 ],
   *   },
   *   page: 2,
   *   limit: 30,
   *   order: 'name ASC'
   * })
   * ```
   * @param {QueryOptions} options
   * @returns array de registros
   */
  static async select(optionsOrSql = {}, args = []) {
    let sql;
    let queryArgs;
    if (typeof optionsOrSql === 'string') {
      // Caso: uso manual del query SQL
      sql = optionsOrSql;
      queryArgs = args;
    } else {
      var _optionsOrSql$where;
      // Caso: uso del objeto QueryOptions
      sql = QueryBuilder.buildSelect(this.tableName, optionsOrSql);
      queryArgs = ((_optionsOrSql$where = optionsOrSql.where) === null || _optionsOrSql$where === void 0 ? void 0 : _optionsOrSql$where.args) || [];
    }
    const result = await this.rawQuery(sql, queryArgs);
    const list = [];
    for (let i = 0; i < result.rows.length; i++) {
      const row = result.rows.item(i);
      list.push(this.databaseToModel(row));
    }
    return list;
  }

  /**
   * const results = await Animal.query()
   *  .select('id, name')
   *  .where('age > ? AND age < ?', [2, 10])
   *  .orderBy('name ASC')
   *  .page(2)
   *  .limit(30)
   *  .get<Animal>();
   * @returns 
   */
  static query() {
    const db = DB.get(this.databaseName);
    return db.table(this.tableName).cursorFactory(this.databaseToModel);
  }

  /**
   * ```js
   * Animal.create({
   *   age: 10,
   *   color: '%Brown%',
   * })
   * ```
   * @param record
   * @returns registro creado
   */
  static async create(record) {
    const databaseValues = this.modelToDatabase(record);
    var sql = QueryBuilder.buildInsert(this.tableName, databaseValues);
    const params = Object.values(databaseValues);
    const result = await this.executeSql(sql, params);
    return await this.find(result.insertId);
  }

  /**
   * ```js
   * Animal.update({
   *   id: 100,
   *   age: 10,
   *   color: '%Brown%',
   * })
   * ```
   * @param record
   * @returns registro actualizado
   */
  static async update(record) {
    const databaseValues = this.modelToDatabase(record);

    // Extrae el valor de "id" y crea un nuevo objeto sin ese dato
    const {
      [this.primaryKey]: id,
      ...props
    } = databaseValues;
    const sql = QueryBuilder.buildUpdate(this.tableName, props, `${this.primaryKey} = ?`);
    const params = Object.values(props);
    await this.executeSql(sql, [...params, id]);
    return await this.find(id);
  }

  /**
   * ```js
   * Animal.destroy(100)
   * ```
   * @param id identificador
   * @returns rowsAffected
   */
  static async destroy(id) {
    const sql = QueryBuilder.buildDelete(this.tableName, `${this.primaryKey} = ?`);
    const result = await this.executeSql(sql, [id]);
    return result.rowsAffected;
  }

  /**
   * ```js
   * Animal.destroyAll()
   * ```
   * @returns rowsAffected
   */
  static async destroyAll() {
    const sql = QueryBuilder.buildDelete(this.tableName);
    const result = await this.executeSql(sql);
    return result.rowsAffected;
  }
  save() {
    const ModelClass = this.constructor;
    const currentInstance = this;
    const id = currentInstance[ModelClass.primaryKey];
    if (id) {
      return ModelClass.update(currentInstance);
    } else {
      return ModelClass.create(currentInstance);
    }
  }
  destroy() {
    const ModelClass = this.constructor;
    const currentInstance = this;
    const id = currentInstance[ModelClass.primaryKey];
    return ModelClass.destroy(id);
  }
}
//# sourceMappingURL=Model.js.map