// Opciones para el QUERY

// Clause Where

// Crea los querys
class QueryBuilder {
  _columns = '*';
  _whereClause = undefined;
  _whereArgs = undefined;
  _orderBy = undefined;
  _limit = undefined;
  _page = undefined;
  constructor(db, tableName) {
    this.db = db;
    this.tableName = tableName;
  }
  select(columns) {
    this._columns = columns;
    return this;
  }
  where(clause, args) {
    this._whereClause = clause;
    this._whereArgs = args;
    return this;
  }
  orderBy(orderBy) {
    this._orderBy = orderBy;
    return this;
  }
  limit(limit) {
    this._limit = limit;
    return this;
  }
  page(page) {
    this._page = page;
    return this;
  }
  cursorFactory(factory) {
    this._cursorFactory = factory;
    return this;
  }

  /** 
   * SELECT
   * 
   * @returns array de registros
   */
  async get(cursorFactory) {
    const sql = QueryBuilder.buildSelect(this.tableName, {
      columns: this._columns,
      where: {
        clause: this._whereClause,
        args: this._whereArgs
      },
      order: this._orderBy,
      limit: this._limit,
      page: this._page
    });
    const result = await this.db.rawQuery(sql, this._whereArgs);
    const rows = [];
    for (let i = 0; i < result.rows.length; i++) {
      const row = result.rows.item(i);
      const factory = cursorFactory ?? this._cursorFactory;
      rows.push(factory ? factory(row) : row);
    }
    return rows;
  }

  /** 
   * INSERT
   * 
   * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
   * @returns insertId
   */
  async insert(record) {
    var sql = QueryBuilder.buildInsert(this.tableName, record);
    const args = Object.values(record);
    const result = await this.db.executeSql(sql, args);
    return result.insertId ?? -1;
  }

  /** 
   * INSERT
   * 
   * @param {string[]} fields ["Column 1", "Column 2"]
   * @param {any[][]} data [
   *  ["foo", "bar"],
   *  ["abc", "def"]
   * ]
   * @returns result
   */
  insertArray(fields, data) {
    const sql = QueryBuilder.buildInsertArray(this.tableName, fields, data);
    // crea una nueva matriz con todos los elementos de las submatrices 
    // concatenados recursivamente hasta la profundidad especificada
    const args = data.flat();
    return this.db.executeSql(sql, args);
  }

  /** 
   * UPDATE
   * 
   * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
   * @returns rowsAffected
   */
  async update(record) {
    const sql = QueryBuilder.buildUpdate(this.tableName, record, this._whereClause);
    const args = Object.values(record);
    const whereArgs = this._whereArgs ?? [];
    const result = await this.db.executeSql(sql, [...args, ...whereArgs]);
    return result.rowsAffected;
  }

  /** 
   * DELETE
   * 
   * @returns rowsAffected
   */
  async delete() {
    const sql = QueryBuilder.buildDelete(this.tableName, this._whereClause);
    const result = await this.db.executeSql(sql, this._whereArgs);
    return result.rowsAffected;
  }

  /**  
   * Creates the "SELECT" sql statement
   * 
   * @param {string} tableName
   * @param {QueryOptions} options
   * @returns string
   */
  static buildSelect(tableName, options) {
    const {
      columns = "*",
      where,
      order,
      limit,
      page
    } = options;
    const sqlParts = ['SELECT', columns, 'FROM', tableName, where && where.clause ? `WHERE ${where.clause}` : '', order ? `ORDER BY ${order}` : '', limit ? `LIMIT ${limit}` : '', page && limit ? `OFFSET ${limit * (page - 1)}` : ''];
    return sqlParts.filter(Boolean).join(' ');
  }

  /** 
   * Creates the "INSERT" sql statement
   * 
   * @param {string} tableName
   * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
   * @returns string
   */
  static buildInsert(tableName, record) {
    const keys = Object.keys(record);
    const columns = keys.join(', ');
    const values = keys.map(() => '?').join(', ');
    return `INSERT INTO ${tableName} (${columns}) VALUES (${values});`;
  }

  /** 
   * Creates the "INSERT" sql statement
   * 
   * @param {string} tableName
   * @param {string[]} fields ["Column 1", "Column 2"]
   * @param {any[][]} data [
   *  ["foo", "bar"],
   *  ["abc", "def"]
   * ]
   * @returns string
   */
  static buildInsertArray(tableName, keys, data) {
    if (keys.length === 0) {
      throw new Error('fields is empty');
    }
    if (data.length === 0) {
      throw new Error('data is empty');
    }
    const columns = keys.join(', ');
    const values = keys.map(() => '?').join(', ');
    const valuesArray = data.map(() => `(${values})`).join(', ');
    return `INSERT INTO ${tableName} (${columns}) VALUES ${valuesArray};`;
  }

  /** 
   * Creates the "Update" sql statement
   * 
   * @param {string} tableName
   * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
   * @param {string} whereClause
   * @returns string
   */
  static buildUpdate(tableName, record, whereClause) {
    // Extrae el valor de "id" y crea un nuevo objeto sin ese dato
    //const { id, ...props } = object
    const values = Object.keys(record).map(k => `${k} = ?`).join(', ');
    const wherePart = whereClause && whereClause.trim().length > 0 ? ` WHERE ${whereClause}` : '';
    return `UPDATE ${tableName} SET ${values}${wherePart};`;
  }

  /**
   * Creates the "DELETE" sql statement
   * 
   * @param {string} tableName
   * @param {string} whereClause
   * @returns string
   */
  static buildDelete(tableName, whereClause) {
    const wherePart = whereClause && whereClause.trim().length > 0 ? ` WHERE ${whereClause}` : '';
    return `DELETE FROM ${tableName}${wherePart};`;
  }

  /**
   * Compila una clausula where
   * 
   * @param {any[]} array [1,3,4,2]
   * @returns (?, ?, ?, ?)
   */
  static whereClausePlaceholders(array) {
    return `(${Array(array.length).fill('?').join(', ')})`;
  }

  /**
   * @param sqlString "valor con 'comillas'"
   * @returns "'valor con \'comillas\''"
   */
  static escapeSqlString(sqlString, escape = '\'') {
    const escapedString = sqlString.replace(new RegExp(escape, 'g'), escape + escape);
    return escape + escapedString + escape;
  }
}
export default QueryBuilder;
//# sourceMappingURL=QueryBuilder.js.map