import SQLite from 'react-native-sqlite-storage';
import type ItMigration from "./ItMigration";
import QueryBuilder from "./QueryBuilder";
export type SqlRequest = {
    sql: string;
    args?: any[];
};
/**
 * Resultado de los querys
 */
export type QueryResult = {
    insertId?: number;
    rowsAffected: number;
    rows: any[];
};
/**
 * Administra la base de datos.
 */
export default class DB {
    private static _instances;
    name: string;
    sqlite?: SQLite.SQLiteDatabase | null;
    /**
     * @param {string} name nombre de la db
     */
    constructor(name: string);
    /**
     * Abre la base de datos
     *
     * @param {string} name nombre de la db
     * @returns {DB} database
     */
    static get(name: string): DB;
    /**
     * Abre la base de datos
     *
     * @param {string} name nombre de la db
     * @returns {DB} database
     */
    static open(name: string): Promise<DB>;
    /**
     * Abre la base de datos
     * @returns {SQLite.SQLiteDatabase}
     */
    open(): Promise<SQLite.SQLiteDatabase>;
    /**
     * Cierra la db
     */
    close(): Promise<void>;
    /**
     * Ejecuta múltiples sentencias SQL en una transacción.
     *
     * ```js
     * db.executeTransaction([
     *   {
     *     sql: "INSERT INTO users (id, name) VALUES (?, ?)",
     *     args: [1, "John"]
     *   },
     *   {
     *     sql: "INSERT INTO users (id, name) VALUES (?, ?)",
     *     args: [2, "Jane"]
     *   },
     * ])
     * ```
     *
     * @param {SqlRequest[]} requests sentencias
     *
     * @returns {SQLite.ResultSet[]} resultados
     */
    executeTransaction(...requests: SqlRequest[]): Promise<SQLite.ResultSet[]>;
    /**
     * Ejecuta sentecias sql.
     *
     * ```js
     * db.rawQuery("SELECT * FROM users WHERE id = ?", [1])
     * db.rawQuery("INSERT INTO users (id, name) VALUES (?, ?)", [1, "John"])
     * ```
     *
     * @param {string} sql sentencia
     * @param {any[]} args parametros
     *
     * @returns {SQLite.ResultSet} resultado
     */
    rawQuery(sql: string, args?: any[]): Promise<SQLite.ResultSet>;
    /**
     * Ejecuta una sentencia SQL con los parámetros dados y devuelve el resultado.
     *
     * ```js
     * db.executeSql("SELECT * FROM users WHERE id = ?", [1])
     * db.executeSql("INSERT INTO users (id, name) VALUES (?, ?)", [1, "John"])
     * ```
     *
     * @param {string} sql La sentencia SQL a ejecutar.
     * @param {any[]} args Los parámetros para la sentencia SQL.
     *
     * @returns {Promise<QueryResult>} El resultado de la consulta en forma de QueryResult.
     */
    executeSql(sql: string, args?: any[]): Promise<QueryResult>;
    /**
     * Obtiene el numero de version de la db
     */
    getVersion(): Promise<number>;
    /**
     * Establece la versión de la base de datos.
     * @param {int} version la nueva versión de la base de datos
     */
    setVersion(version: number): Promise<void>;
    /**
     * Inicializa el scheme de la base de datos.
     */
    migrate(migration: ItMigration, version: number): Promise<void>;
    /**
     * Obtiene un QueryBuilder
     *
     * @param tableName nombre de la tabla
     * @returns
     */
    table(tableName: string): QueryBuilder;
}
//# sourceMappingURL=DB.d.ts.map