import SQLite from 'react-native-sqlite-storage';
import { type QueryResult } from './DB';
import QueryBuilder, { type QueryOptions } from './QueryBuilder';
export default class Model {
    static get databaseName(): string;
    static get tableName(): string;
    static get primaryKey(): string;
    constructor(props?: any);
    static databaseToModel(databaseValues: any): Model;
    static modelToDatabase(props: any): any;
    /**
     * ```js
     * Animal.executeSql("SELECT * FROM tb_animal WHERE id = ?", [7])
     * ```
     * @return {QueryResult} result
     */
    static executeSql(sql: string, args?: any[]): Promise<QueryResult>;
    static rawQuery(sql: string, args?: any[]): Promise<SQLite.ResultSet>;
    /**
     * ```js
     * Animal.findBy<Animal>("color", "LIKE", '%Brown%')
     * ```
     * @param column mombre de la columna
     * @param op operador: =, <>, <, <=, >, >=, LIKE
     * @param value qriterio de busqueda
     * @return registro
     */
    static findBy<T extends Model>(column: string, op: string, value: string): Promise<T | undefined>;
    /**
     * ```js
     * Animal.find(1)
     * ```
     * @param id identificador del registro
     * @return registro
     */
    static find<T extends Model>(id: any): Promise<T | undefined>;
    /**
     * ```js
     * Animal.first()
     * ```
     * @return primer registro
     */
    static first<T extends Model>(): Promise<T | null>;
    /**
     * ```js
     * Animal.last()
     * ```
     * @return último registro
     */
    static last<T extends Model>(): Promise<T | null>;
    /**
     * ```js
     * Animal.count()
     * ```
     * @return numero de registros
     */
    static count(): Promise<number>;
    /**
     * ```js
     * Animal.all()
     * ```
     * @return array con todos los registros
     */
    static all<T extends Model>(): Promise<T[]>;
    /**
     * ```js
     * Animal.query({
     *   columns: 'id, name',
     *   where: {
     *     clause: 'age > ? AND age < ?',
     *     args: [ 2, 10 ],
     *   },
     *   page: 2,
     *   limit: 30,
     *   order: 'name ASC'
     * })
     * ```
     * @param {QueryOptions} options
     * @returns array de registros
     */
    static select<T extends Model>(optionsOrSql?: QueryOptions | string, args?: any[]): Promise<T[]>;
    /**
     * const results = await Animal.query()
     *  .select('id, name')
     *  .where('age > ? AND age < ?', [2, 10])
     *  .orderBy('name ASC')
     *  .page(2)
     *  .limit(30)
     *  .get<Animal>();
     * @returns
     */
    static query(): QueryBuilder;
    /**
     * ```js
     * Animal.create({
     *   age: 10,
     *   color: '%Brown%',
     * })
     * ```
     * @param record
     * @returns registro creado
     */
    static create<T extends Model>(record: any): Promise<T | undefined>;
    /**
     * ```js
     * Animal.update({
     *   id: 100,
     *   age: 10,
     *   color: '%Brown%',
     * })
     * ```
     * @param record
     * @returns registro actualizado
     */
    static update<T extends Model>(record: any): Promise<T | undefined>;
    /**
     * ```js
     * Animal.destroy(100)
     * ```
     * @param id identificador
     * @returns rowsAffected
     */
    static destroy(id: any): Promise<number>;
    /**
     * ```js
     * Animal.destroyAll()
     * ```
     * @returns rowsAffected
     */
    static destroyAll(): Promise<number>;
    save<T extends Model>(): Promise<T | undefined>;
    destroy(): Promise<number>;
}
//# sourceMappingURL=Model.d.ts.map