import DB, { type QueryResult } from "./DB";
export type QueryOptions = {
    columns?: string;
    page?: number;
    limit?: number;
    where?: Where;
    order?: string;
};
export type Where = {
    clause?: string;
    args?: any[];
};
declare class QueryBuilder {
    db: DB;
    tableName: string;
    private _columns;
    private _whereClause;
    private _whereArgs;
    private _orderBy;
    private _limit;
    private _page;
    private _cursorFactory?;
    constructor(db: DB, tableName: string);
    select(columns: string): this;
    where(clause: string, args: any[]): this;
    orderBy(orderBy: string): this;
    limit(limit: number): this;
    page(page: number): this;
    cursorFactory(factory: (row: any) => any): this;
    /**
     * SELECT
     *
     * @returns array de registros
     */
    get<V>(cursorFactory?: (row: any) => V): Promise<V[]>;
    /**
     * INSERT
     *
     * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
     * @returns insertId
     */
    insert(record: any): Promise<number>;
    /**
     * INSERT
     *
     * @param {string[]} fields ["Column 1", "Column 2"]
     * @param {any[][]} data [
     *  ["foo", "bar"],
     *  ["abc", "def"]
     * ]
     * @returns result
     */
    insertArray(fields: string[], data: any[][]): Promise<QueryResult>;
    /**
     * UPDATE
     *
     * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
     * @returns rowsAffected
     */
    update(record: any): Promise<number>;
    /**
     * DELETE
     *
     * @returns rowsAffected
     */
    delete(): Promise<number>;
    /**
     * Creates the "SELECT" sql statement
     *
     * @param {string} tableName
     * @param {QueryOptions} options
     * @returns string
     */
    static buildSelect(tableName: string, options: QueryOptions): string;
    /**
     * Creates the "INSERT" sql statement
     *
     * @param {string} tableName
     * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
     * @returns string
     */
    static buildInsert(tableName: string, record: any): string;
    /**
     * Creates the "INSERT" sql statement
     *
     * @param {string} tableName
     * @param {string[]} fields ["Column 1", "Column 2"]
     * @param {any[][]} data [
     *  ["foo", "bar"],
     *  ["abc", "def"]
     * ]
     * @returns string
     */
    static buildInsertArray(tableName: string, keys: string[], data: any[][]): string;
    /**
     * Creates the "Update" sql statement
     *
     * @param {string} tableName
     * @param {any} record { "Column 1": "foo", "Column 2": "bar" }
     * @param {string} whereClause
     * @returns string
     */
    static buildUpdate(tableName: string, record: any, whereClause?: string): string;
    /**
     * Creates the "DELETE" sql statement
     *
     * @param {string} tableName
     * @param {string} whereClause
     * @returns string
     */
    static buildDelete(tableName: string, whereClause?: string): string;
    /**
     * Compila una clausula where
     *
     * @param {any[]} array [1,3,4,2]
     * @returns (?, ?, ?, ?)
     */
    static whereClausePlaceholders(array: any[]): string;
    /**
     * @param sqlString "valor con 'comillas'"
     * @returns "'valor con \'comillas\''"
     */
    static escapeSqlString(sqlString: string, escape?: string): string;
}
export default QueryBuilder;
//# sourceMappingURL=QueryBuilder.d.ts.map