import DB, { type QueryResult } from './DB';
export default class Schema {
    db: DB;
    constructor(db: DB);
    execSQL(sql: string): Promise<QueryResult>;
    /**
     * Crea la tabla
     * @param tableName
     * @param closure
     * @returns
     */
    create(tableName: string, closure: (table: TableSchema) => void): Promise<TableSchema>;
    /**
     * Agrega las columnas nuevas que no existen fisicamente en la tabla.
     * @param tableName
     * @param closure
     */
    alter(tableName: string, closure: (table: TableSchema) => void): Promise<TableSchema>;
    /**
     * Crea o modifica la tabla
     * @param tableName
     * @param closure
     * @returns
     */
    createOrAlter(tableName: string, closure: (table: TableSchema) => void): Promise<TableSchema>;
    /**
     * Elimina la tabla
     */
    drop(tableName: string): Promise<void>;
    /**
     * Valida si existe una tabla en la base de datos.
     */
    hasTable(tableName: string): Promise<boolean>;
    getColumnNames(tableName: string): Promise<string[]>;
    /**
     * Obtiene un lista de columnas que ha un no exiten fisicamente en la tabla.
     */
    getMissingColumns(table: TableSchema): Promise<Array<Column>>;
    /**
     * Crea una nueva tabla con la misma estructura que una tabla existente, incluyendo llaves foráneas, pero sin datos.
     * @param originalTableName Nombre de la tabla existente
     * @param newTableName Nombre de la nueva tabla
     */
    duplicateTableStructure(originalTableName: string, newTableName: string): Promise<void>;
    /**
     * Copia los datos de una tabla a otra.
     * @param sourceTableName Nombre de la tabla de origen.
     * @param destinationTableName Nombre de la tabla de destino.
     */
    copyTableData(sourceTableName: string, destinationTableName: string): Promise<QueryResult>;
}
export declare class TableSchema {
    name: string;
    isCreated: boolean;
    columns: Column[];
    constraints: Constraint[];
    indexs: Index[];
    constructor(name: string);
    addColumn(columname: string, type: string): Column;
    addConstraint(): Constraint;
    addIndex(indexname: string): Index;
    /** Incrementing ID to the table (primary key)  */
    increments(column: string): Column;
    /** INTEGER equivalent to the table  */
    integer(column: string): Column;
    /** FLOAT equivalent to the table  */
    float(column: string): Column;
    /** DECIMAL equivalent with a precision and scale  */
    real(column: string): Column;
    /** VARCHAR equivalent column  */
    text(column: string): Column;
    /** Binary equivalent column  */
    blob(column: string): Column;
    toString(): string;
    foreign(columnname: string): Constraint;
    index(indexname: string): Index;
}
/**
 * id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL DEFAUL '' +
 */
export declare class Column {
    name: string;
    type: string;
    private _primaryKey;
    private _autoIncrement;
    private _nullable;
    private _defaultVal;
    constructor(name: string, type: string);
    primaryKey(): Column;
    autoIncrement(): Column;
    default(defaultVal: any): Column;
    nullable(): Column;
    toString(): string;
}
/**
 * CREATE UNIQUE INDEX IF NOT EXISTS 'index_picking'
 * ON 'tb_picking' ('folio')
 */
export declare class Index {
    name: string;
    tablename: string;
    private _unique;
    private _columns;
    constructor(name: string, tablename: string);
    unique(...columns: string[]): void;
    columns(...columns: string[]): void;
    toString(): string;
    private escapeSQLString;
}
/**
 * FOREIGN KEY(id_inventario) REFERENCES tb_inventario(id) ON DELETE CASCADE
 */
export declare class Constraint {
    private _foreign;
    private _references;
    private _table;
    private _onDelete;
    foreign(...key: string[]): Constraint;
    references(...key: string[]): Constraint;
    on(table: string): Constraint;
    onDelete(on?: string): void;
    toString(): string;
}
//# sourceMappingURL=Schema.d.ts.map